'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Created by jcox on 6/6/16.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _config = require('./lib/config');

var _configMapper = require('./lib/configMapper');

var _configMapper2 = _interopRequireDefault(_configMapper);

var _casperManager = require('./lib/casperManager');

var _casperManager2 = _interopRequireDefault(_casperManager);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CasperTestRunner = function () {
    /**
     * Runs casper tests in a given dist folder transpiled from es6 using babel
     * 
     * @param {Object} casper The casperjs instance
     * @param {Object} phantom The phantomjs instance
     * @param {string} testPath The relative path to the transpiled casper tests
     * @param {Object<number, string>} browser The browser width, height, and userAgent for the test
     * @param {string} startUrl The url to start the test
     */

    function CasperTestRunner(casper, phantom, testPath, browser, startUrl) {
        _classCallCheck(this, CasperTestRunner);

        this.casper = casper;
        this.phantom = phantom;
        this.cm = new _casperManager2.default(casper, phantom);
        this.testPath = testPath;
        this.browser = _configMapper2.default.map(_config.browserDefaults, browser);
        this.startUrl = startUrl || _config.defaultStartUrl;
        this.testFiles = [];
        this.tests = [];
        this.fs = require('fs');
        this.scriptName = this.fs.absolute(require("system").args[3]);
        this.scriptDirectory = this.scriptName.substring(0, this.scriptName.lastIndexOf('/'));
        this.setCasperOptions({ viewportSize: { width: 1024, height: 2000 }, waitTimeout: 50000 });
    }

    /**
     * Sets the casper browser options
     * @returns {void}
     */


    _createClass(CasperTestRunner, [{
        key: 'setupBrowser',
        value: function setupBrowser() {
            this.casper.viewport(this.browser.width, this.browser.height);
            this.casper.userAgent(this.browser.userAgent);
        }

        /**
         * Sets casper options in the casper.options object
         * 
         * @param {Object} options The options to set in casper
         * 
         * @returns {void}
         */

    }, {
        key: 'setCasperOptions',
        value: function setCasperOptions(options) {
            for (var k in options) {
                this.casper.options[k] = options[k];
            }
        }

        /**
         * Gets all of the test function names from a file
         * 
         * @param {string} file The contents of a test class file
         * 
         * @returns {Array} The tests found in the file
         */

    }, {
        key: 'getTests',
        value: function getTests(file) {
            var tests = [],
                regex = /\@test[\s\S]+?(?=key:\s?[\"\'])key:\s?[\"\']([\w\d-_]+)[\"\']/gi,
                result = void 0;

            while (result = regex.exec(file)) {
                if (result[1]) {
                    tests.push(result[1]);
                }
            }

            return tests;
        }

        /**
         * Retrieves all of the test files and puts all of their information into the testFiles array
         * 
         * @param {string} path The path to the tests
         */

    }, {
        key: 'findTestFiles',
        value: function findTestFiles(path) {
            var list = this.fs.list(fs.workingDirectory + '/' + path),
                file = void 0,
                fileData = void 0,
                className = void 0,
                tests = void 0;

            for (var i = 0; i < list.length; i++) {
                file = path + list[i];

                // ignore current and parent directories
                if (list[i] == '.' || list[i] == '..') {
                    continue;
                } else if (fs.isDirectory(file)) {
                    this.findTestFiles(file + '/');
                } else if (this.fs.isFile(file) && list[i].indexOf('.test.js') != -1) {
                    fileData = this.getFileData(file);
                    if (fileData) {
                        this.testFiles.push(fileData);
                    }
                }
            }
        }

        /**
         * Parses the test file, and and gets the class name and the test method names
         *
         * @param {string} file The path to the test file
         *
         * @returns {*}
         */

    }, {
        key: 'getFileData',
        value: function getFileData(file) {
            this.casper.echo(file);
            var fileData = this.fs.read(file);
            var className = fileData.match(/exports.default\s\=\s([\w\d-_]+);/);
            var tests = this.getTests(fileData);
            if (className) {
                return { file: file, className: className[1], tests: tests };
            } else {
                return null;
            }
        }

        /**
         * Runs a test with the given test params
         * 
         * @param {Object<string, Array>} testObj 
         * 
         * @returns {void}
         */

    }, {
        key: 'runTest',
        value: function runTest(testObj) {
            var _this = this;

            var mod = require(this.scriptDirectory + '/' + testObj.file);

            this.casper.test.begin(mod.description, testObj.tests.length, function (test) {

                _this.cm.setTestObject(test);

                var inst = new mod.default(_this.cm);
                _this.cm.setDebug(false, mod.description.replace(/\s/g, ''));
                _this.casper.start(_this.startUrl, function () {
                    _this.setupBrowser();
                });

                if (inst.setUpBefore) {
                    _this.casper.then(function () {
                        inst.setUpBefore();
                    });
                }

                var _loop = function _loop(i) {
                    if (inst.setUp) {
                        _this.casper.then(function () {
                            inst.setUp();
                        });
                    }
                    _this.casper.then(function () {
                        inst[testObj.tests[i]](test);
                    });
                    if (inst.tearDown) {
                        _this.casper.then(function () {
                            inst.tearDown();
                        });
                    }
                };

                for (var i = 0; i < testObj.tests.length; i++) {
                    _loop(i);
                }

                if (inst.tearDownAfter) {
                    _this.casper.then(function () {
                        inst.tearDownAfter();
                    });
                }

                _this.casper.run(function () {
                    test.done();
                    setTimeout(function () {
                        _this.phantom.exit();
                    }, 10);
                });
            });
        }

        /**
         * Checks to see if the testFiles flag has been passed and puts them in the testFiles array if they have
         *
         * @returns {boolean}
         */

    }, {
        key: 'getCliTestFiles',
        value: function getCliTestFiles() {
            var self = this;

            if (!this.casper.cli.has('testFiles')) {
                return false;
            } else {
                this.casper.cli.get('testFiles').split(',').forEach(function (fileName) {
                    var fileData = self.getFileData(self.testPath + fileName);
                    if (fileData) {
                        self.testFiles.push(fileData);
                    }
                });

                return true;
            }
        }

        /**
         * Runs all of the test in the given test folder
         * 
         * @returns {void}
         */

    }, {
        key: 'run',
        value: function run() {
            if (!this.getCliTestFiles()) {
                // get all the test information
                this.findTestFiles(this.testPath);
            }

            // run all the tests
            for (var i = 0; i < this.testFiles.length; i++) {
                this.runTest(this.testFiles[i]);
            }
        }
    }]);

    return CasperTestRunner;
}();

exports.default = CasperTestRunner;