'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Created by jcox on 6/3/16.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

var _q = require('q');

var Q = _interopRequireWildcard(_q);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CasperManager = function () {
    /**
     * Class to manage casper and DOM interactions
     * @param casper The casper object passed as global
     * @param phantom The phantom object passed as global
     */

    function CasperManager(casper, phantom) {
        _classCallCheck(this, CasperManager);

        this.casper = casper;
        this.phantom = phantom;

        this.failCount = 0;
    }

    /**
     * Set debug options in casper
     *
     * @param {Boolean} debug Whether to have debug mode turned on
     * @param {string} baseName The folder name for logging output and image captures
     *
     * @returns {void}
     */


    _createClass(CasperManager, [{
        key: 'setDebug',
        value: function setDebug(debug, baseName) {
            var self = this;

            if (this.casper.cli.get('trace') == true) {
                debug = true;
            }
            this.casper.on('page.error', function (msg, trace) {
                if (debug) {
                    self.casper.echo('Error: ' + msg, 'ERROR\n');
                    for (var i in trace) {
                        self.casper.echo(JSON.stringify(trace[i]));
                    }
                }
            });
            this.casper.on('remote.message', function (msg) {
                if (debug) {
                    self.casper.echo('remote console message: ' + msg);
                }
            });
            this.casper.test.on('fail', function () {
                self.failCount++;
                self.casper.capture('screenshots/' + baseName + '/' + baseName + '_fail_' + self.failCount + '.png');
                self.casper.echo("Failed - image captured");
            });
        }

        /**
         * Clicks an element in the DOM after waiting for the element to be there
         *
         * @param {string} selector The selector of the element to be clicked
         *
         * @returns {void}
         */

    }, {
        key: 'click',
        value: function click(selector) {
            var self = this;

            this.waitForSelector(selector, function () {
                self.casper.click(selector);
            });
        }

        /**
         * Retrieves the text from an element
         *
         * @param {string} selector The selector of the element from which to get the text
         *
         * @returns {*} Promise that resolves with the text
         */

    }, {
        key: 'getText',
        value: function getText(selector) {
            var self = this,
                deferred = Q.defer();

            this.waitForSelector(selector, function () {
                deferred.resolve(self.casper.fetchText(selector));
            });

            return deferred;
        }

        /**
         * Waits for a selector to be available on the page
         *
         * @param {string} selector The selector for which to wait
         * @param {Function} args The rest of the arguments. ie callback, onTimeout, timeout
         *
         * @returns {void}
         */

    }, {
        key: 'waitForSelector',
        value: function waitForSelector(selector) {
            for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                args[_key - 1] = arguments[_key];
            }

            var self = this;

            this.casper.then(function () {
                var _self$casper;

                (_self$casper = self.casper).waitForSelector.apply(_self$casper, [selector].concat(args));
            });
        }

        /**
         * Wrapper for wait methods to make them psuedo synchronous
         *
         * @param method The wait method to call
         * @param selector The selector to wait for
         * @param args The rest of the arguments
         *
         * @returns {void}
         */

    }, {
        key: 'waitFor',
        value: function waitFor(method, selector) {
            for (var _len2 = arguments.length, args = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
                args[_key2 - 2] = arguments[_key2];
            }

            if (this.casper[method] && method.indexOf('wait') > -1) {
                this.casper.then(function () {
                    var _self$casper2;

                    (_self$casper2 = self.casper)[method].apply(_self$casper2, [selector].concat(args));
                });
            } else {
                throw new Error("Param 'method' must be a casperjs wait function.");
            }
        }

        /**
         * Waits until selector no longer exists in the DOM
         *
         * @param {string} selector The selector of the element to wait for
         * @param args All of the other arguments. ie callback, onTimeout, and timeout
         *
         * @returns {void}
         */

    }, {
        key: 'waitWhileSelector',
        value: function waitWhileSelector(selector) {
            for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
                args[_key3 - 1] = arguments[_key3];
            }

            var self = this;

            this.waitForSelector(selector, function () {
                var _self$casper3;

                (_self$casper3 = self.casper).waitWhileSelector.apply(_self$casper3, [selector].concat(args));
            });
        }

        /**
         * Evaluates an expression in the current page DOM context
         *
         * @param {Object} params The params to be used in the evaluate callback
         * @param {Function} cb The script to execute against the DOM
         * @param {string} waitForSelector Optional selector to wait for before executing the callback
         *
         * @returns {*|promise}
         */

    }, {
        key: 'evaluate',
        value: function evaluate(params, cb, waitForSelector) {
            var self = this,
                deferred = Q.defer();

            if (waitForSelector) {
                this.waitForSelector(waitForSelector, function () {
                    var response = self.casper.evaluate(cb, params);
                    deferred.resolve(response);
                });
            } else {
                this.casper.then(function () {
                    var response = self.casper.evaluate(cb, params);
                    deferred.resolve(response);
                });
            }

            return deferred.promise;
        }

        /**
         * Types text into an input field
         *
         * @param {string} selector The selector of the field to type into
         * @param {string} keys The text to type
         */

    }, {
        key: 'sendKeys',
        value: function sendKeys(selector, keys) {
            var self = this;

            this.waitForSelector(selector, function () {
                self.casper.sendKeys(selector, keys);
            });
        }

        /**
         * Helper method for casper element getters
         * 
         * @param {string} selector The element selector
         * @param {Function} getter the casper method to call
         * @param {string?} attr (Optional) The attribute to get from the element
         * @returns {*|promise}
         */

    }, {
        key: 'elementGetHelper',
        value: function elementGetHelper(selector, getter, attr) {
            var deferred = Q.defer();

            this.waitForSelector(selector, function () {
                var arr = void 0;

                if (attr) {
                    arr = getter(selector, attr);
                } else {
                    arr = getter(selector);
                }

                deferred.resolve(arr.length == 1 ? arr[0] : arr);
            });

            return deferred.promise;
        }

        /**
         * Gets an attibute from all elements matching the selector
         *
         * @param {string} selector The element selector
         * @param {string} attr The attribute to get
         * 
         * @returns {*|promise}
         */

    }, {
        key: 'getAttribute',
        value: function getAttribute(selector, attr) {
            return this.elementGetHelper(selector, this.casper.getElementsAttribute, attr);
        }

        /**
         * Retrieves the boundaries (coordinates) of the elements with the given selector
         *
         * @param {string} selector The element selector
         * 
         * @returns {*|promise}
         */

    }, {
        key: 'getBounds',
        value: function getBounds(selector) {
            return this.elementGetHelper(selector, this.casper.getElementsBounds);
        }

        /**
         * Retrieves all of the info of the elements with the given selector
         *
         * @param {string} selector The element selector
         *
         * @returns {*|promise}
         */

    }, {
        key: 'getInfo',
        value: function getInfo(selector) {
            return this.elementGetHelper(selector, this.casper.getElementsInfo);
        }

        /**
         * Retrieves the casper instance
         * 
         * @returns {*}
         */

    }, {
        key: 'getCasper',
        value: function getCasper() {
            return this.casper;
        }

        /**
         * Retrieves the phantom instance
         * 
         * @returns {*}
         */

    }, {
        key: 'getPhantom',
        value: function getPhantom() {
            return this.phantom;
        }

        /**
         * Sets the test object from casperjs
         *
         * @param {Object} test The casperjs test object
         *
         * @returns {void}
         */

    }, {
        key: 'setTestObject',
        value: function setTestObject(test) {
            this.test = test;
        }

        /**
         * Uses the casperjs test.comment function if available, or uses casper.echo if not, and is beautifully wrapped in a casper.then
         *
         * @param {String} text The text to write to the console
         *
         * @returns {void}
         */

    }, {
        key: 'comment',
        value: function comment(text) {
            var _this = this;

            this.casper.then(function () {
                if (_this.test) {
                    _this.test.comment(text);
                } else {
                    _this.casper.echo('# ' + text);
                }
            });
        }

        /**
         * casper.echo wrapper that places the echo in a casper.then
         *
         * @param {String} text The text to write to the console
         *
         * @returns {void}
         */

    }, {
        key: 'echo',
        value: function echo(text) {
            var _this2 = this;

            this.casper.then(function () {
                _this2.casper.echo(text);
            });
        }

        /**
         * Wrapper for all casperjs asserts, because I am too lazy to write them all out
         *  Wraps method in a casper.then and spreads arguments out into the actual method
         *
         * @param method The assert method to call
         * @param args Array of arguments converted from actual arguments (Thank you es6)
         */

    }, {
        key: 'assert',
        value: function assert(method) {
            var _this3 = this;

            for (var _len4 = arguments.length, args = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
                args[_key4 - 1] = arguments[_key4];
            }

            if (this.test[method] && method.indexOf('assert') > -1) {
                this.casper.then(function () {
                    var _test;

                    (_test = _this3.test)[method].apply(_test, args);
                });
            } else {
                throw new Error("Param method must be a valid casperjs test assert method.");
            }
        }
    }]);

    return CasperManager;
}();

exports.default = CasperManager;